#ifndef __XVAL_H
#define __XVAL_H 1
/* Expectationvalues etc. of EF */

#include "defs.h"

#ifndef MAXx
#define MAXx 200l	/* max. Anzahl auszugebener Koeffizienten */
#endif

extern ULong	x_max;		/* max. anzahl koeff. 0..MAXx	*/

/* return a unique(?) list of counted different pairs = ising energy index
   uu+dd ud+du ..+33 .3+3. .u+.d+u.+d.+3u+3d+u3+d3  (16pairs -> 5sympairs)
   the list is allocated and must be freed after use
   mode==0: output list, free list, return 0
   mode==1: return list
   buf = return space OR NULL for malloc
 */
int *get_dkorr(tbase b1, int mode, int *buf); /* Apr06, v2.25 */

/* -------------------------------------------------------------------- */
/* --------       Analyse des Eigenvectors                  ----------- */
/* -------------------------------------------------------------------- */
/* Koeff. x_i sortieren, groesster x_i => r[MAXx-1]
 * mode=0 maximum values  
 * mode=1 minimum values
*/
int x_out(FILE *f,tvec *v0,int mode);

/* ---- Anzahl verschiedener Koeffizienten bestimmen  --------- */
int x_sort(FILE *f,tvec *v0,tvec *v1);

cplx wh(tvec *v0);			/* Erwartungswert <H> */

double whq(tvec *v0,cplx rh);  /* Fehler Erwartungswert (H*v-h*v)/h */

/* mittelwert <v1|T*v0>/<v0|v0> (v1=v0) und abweichung (T|v0>-<T>v0)^2 
 *   |k> = q sum_R^l exp(i*k*R) T_R |v>   ( Tx_R |x> = |x+R> ) 
 *       = q sum_m^l exp(i*k*m) T^m |v>   - Fouriertransformation k=2*Pi/l*k
 *   rk  = <v|k>			  q=1/sqrt(sum_m^l exp(i*k*m)^2)=2/l
 */ 
cplx wop_k(tvec *v0,tvec *v1,int ip);
    
cplx wop_t(tvec *v0,tvec *v1,int ip);
 
cplx wop_ud(tvec *v0,tvec *v1); /* v1=T*v0  u<->d */

zahl wss(tvec *v0,tvec *v1); /* SS=1/2*(SpSm+SmSp)+S0^2 ss=s(s+1) */

cplx wjj(tvec *v0,tvec *v1); /* Hubbard SuperSpin Lit.? */

/*  ------------------------------------------------------------
 *  use this function to calculate expectationvalues <v0|op|v1>
 *  of your own operator void op(void *param,int num_sites,int *sites,hbuf)
 *  with hbuf=op(kfg,param)
 *  example_call: wopij(&op_sisj,&para,2,sites,v0,v1);
 *  =>  (void (*)(void *,int,int *,thbuf *)) &op_sisj
 */
#if 0
cplx wopij( void (*op)(void *,int,int,thbuf *),
   void *para,int iw1,int iw2,tvec *v0,tvec *v1);           /* old */
#endif
cplx wop( void (*op)(void *,int,int *,thbuf *),
   void *para,int num_sites,int *sites,tvec *v0,tvec *v1);  /* new */
      
/*
 *  example operator (nu+nd)(i)*(nu+nd)(j) = op_ninj for wop calculation
 *  decouples operator from vector and symmetry 
 *  and makes the program more flexible for users to create new operators
 *  call_by wop(&op_ninj,NULL,2,iw,v0,v1);
 *   this operator is not symmetric against spin inversion (u <=> d) ???
 *   so we have wrong results for SBase + Sud + nu==nd
 */
void op_ninj  (void *para,int ns,int *iw,thbuf *hbuf);
void op_nisnjs(void *para,int ns,int *iw,thbuf *hbuf);

/*  --- occupation operator ---
 *  example operator n(i,s) = op_nis for wop calculation
 *  decouples operator from vector and symmetry
 *  and makes the program more flexible for users to create new operators
 *  call_by wop(&op_nis,(void *)&s1,1,&site,v0,v1);
 *   this operator is not symmetric against spin inversion (u <=> d) ???
 *   so we have wrong results for SBase + Sud + nu==nd
 *   para is "1" for n(i,up), "2" for n(i,down) and "3" for nu*nd=ud
 */
void op_nis( void *para,int ns,int *iw,thbuf *hbuf); /* nis */
void op_zi(  void *para,int ns,int *iw,thbuf *hbuf); /* Zi */
void op_zizj(void *para,int ns,int *iw,thbuf *hbuf); /* ZiZj */
void op_xixj(void *para,int ns,int *iw,thbuf *hbuf); /* XiXj */
void op_sisj(void *para,int ns,int *iw,thbuf *hbuf); /* SiSj */
void op_sisjsksl(void *para,int ns,int *iw,thbuf *hbuf); /* see xval.c */
void op_cicj    (void *para,int ns,int *iw,thbuf *hbuf); /* see xval.c */
void op_cicjckcl(void *para,int ns,int *iw,thbuf *hbuf); /* see xval.c */

/* 1-Marshall-weight (weight of states breaking the marshall rule) */
cplx mrule(tvec *v0, int *sl, int num_sl);

/* evl. zi()-fkt fuer spin1 in modeldef.h definieren! (allgemeiner) */
/* only for dimer model, spin 1 chains , spin-1 = 2 Spin-1/2 = (4n+2)+(4n+3)
   O(z,pi,i,j)=<v1|Szi*exp(sum(i*Pi*Szk,k=i+1..j))*Szj|v0> |Si|=1*/
int stringorder(FILE *f1,tvec *v0);

/* only for spin 1 chains , spin-1 = 2 Spin-1/2 = (4n+2)+(4n+3)
   O(z,pi,i,j)=<v1|Szi*exp(sum(i*Pi*Szk,k=i+1..j))*Szj|v0> |Si|=1*/
int stringorder2(FILE *f1,tvec *v0);

/* Z=<> */
#ifdef SBase
cplx wzijk(int iw1, ...);
#else
cplx wzijk(int iw1,int iw2,int iw3,tvec *v0,tvec *v1);
#endif

#if defined(N1) && defined(N2) && N1>1 
cplx wdx(tvec *v0);
cplx wdy(tvec *v0);
cplx wc(tvec *v0);
cplx wz(tvec *v0);
int  wtatb(cplx *rta, cplx *rtb, tvec *v0);
#endif	/* N1>1 */

#endif
